function varargout = AnalyzeLogsGUI_SC(varargin)

% Edited by Joshua Mayourian on 4/13/20
% Edited by Serena Chang on 4/4/21

% The following code helps initiate analyze_logs_plus, which analyzes logs
% produced by the PostTracking.vi LabVIEW program

% analyze_logs_plus then calculates force, stress and other parameters are calculated

% Given a tissue number and a table of tissue parameters for each
% tissue, the script finds all logs for that tissue number, given
% that they follow a common format. The format is:
%
% Tn @ P hz XXX.dat 
%
% where n is the tissue number given, P is the pacing frequency given
% and X is any digit from 0 to 9. P may be either an integer (ex. 3)
% or a number with one to three digits after a decimal point (ex. 3.4,
% 0.435). If P is less than 1, there must be a leading zero before the
% decimal point.

% Usage: 
% 1) Select directory where tissue and post-height files (named as
% SummaryX.csv, where X is the number of tissue culture days). Day Number
% should then automatically fill in
% 2) Select tissue, where tissue file format is described above
% 3) Omit/include frequencies using format described above
% 4) Change filters as needed, and select buttons as needed
% 5) Insert post radius and diastolic distance (ask Camille for default
% values)
% 6) Hit Run Analysis

% GUI initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @AnalyzeLogsGUI_OpeningFcn, ...
                   'gui_OutputFcn',  @AnalyzeLogsGUI_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% Fills in key values, then runs analyze_logs_plus
function pushbutton2_Callback(hObject, eventdata, handles)
global summaryname
tnum_str = get(handles.selectTissue,'String'); % Tissue number (string)
tval = get(handles.selectTissue,'Value'); % Tissue number (value)
tnum = str2num(tnum_str{tval}(2:end));
p_order = str2num(get(handles.p_order,'String')); % Polynomial order
w_size = str2num(get(handles.f_size,'String')); % Frame size
etime = str2num(get(handles.edit_etime,'String')); % End time
stime = str2num(get(handles.edit_stime,'String')); % Start time

if get(handles.checkbox_spikeremoval,'Value') == 1 % Spike removal
    spike_thresh = get(handles.spikesens,'Value');
else
    spike_thresh = 0;
end

if get(handles.checkbox_asy,'Value') == 1 % Autoscale
    asy = 1;
else
    asy = 0;
end

omitfr = get(handles.edit_omitfr,'String'); % Omit frequencies
inclfr = get(handles.edit_inclfr,'String'); % Include frequencies
plot_raw = get(handles.checkbox_plotraw,'Value'); % Show unfiltered data
save_ftfig = get(handles.checkbox_saveftfig, 'Value'); % Save force trace figure
save_ftdata = get(handles.checkbox_saveftdata,'Value'); % Save force time data

peak_thresh = get(handles.slider_pdt,'Value'); % Peak detection threshold
postrad = str2num(get(handles.edit_postrad,'String')); % Post radius
diastolicdist = str2num(get(handles.edit16,'String')); % Diastolic distance
daynum = str2num(get(handles.edit_dnum,'String')); % Day number
pda = get(handles.checkbox_pda,'Value'); % Post deflection analysis

% Make sure post radius and diastolic distance values are inputted
if postrad == 0
    fprintf('Please update post radius \n');
    return
end

if diastolicdist == 0
    fprintf('Please update diastolic distance \n');
    return
end

% Run analyze_logs_plus
analyze_logs_plus_r6_SC(summaryname, daynum, tnum, p_order,w_size,spike_thresh,...
    peak_thresh,postrad,pda, asy, omitfr,inclfr,stime,etime,plot_raw,diastolicdist,save_ftfig,save_ftdata);
guidata(hObject,handles)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The remaining code just helps the GUI run
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function AnalyzeLogsGUI_OpeningFcn(hObject, eventdata, handles, varargin)
handles.output = hObject;
guidata(hObject, handles);

function varargout = AnalyzeLogsGUI_OutputFcn(hObject, eventdata, handles) 
varargout{1} = handles.output;

function selectTissue_Callback(hObject, eventdata, handles)
function selectTissue_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Involves directory selection
function selectdirectory_Callback(hObject, eventdata, handles)
global summaryname
addpath(genpath(pwd))
directory = uigetdir('','Select folder containing log files');
cd(directory);
d = dir('*.dat');

files = extractfield(d,'name');
tnames = {};
for i = 1:length(files);
    a = strsplit(files{i});
    tnames{i} = a{1};
end
tnums = unique(tnames);
set(handles.selectTissue,'String', tnums);
d2 = dir('*.csv');
fnames = extractfield(d2,'name');
summary = cell2mat(strfind(fnames,'summary'));
ind = find(summary == 1);
if isempty(ind)
    summary = cell2mat(strfind(fnames,'Summary'));
    ind = find(summary == 1);
end
if isempty(ind)
    error('No Summary File Found')
end

summaryfile = cell2mat(fnames(ind));
summaryname = strsplit(summaryfile,'.csv');
summaryname = summaryname{1};
dnum = summaryname(isstrprop(summaryname,'digit'));

set(handles.edit_dnum, 'String', dnum);
set(handles.textsummary,'String',summaryname);

guidata(hObject,handles);

% Peak detection threshold
function edit_pdt_Callback(hObject, eventdata, handles)
pdt = str2num(get(handles.edit_pdt,'String'));
if (pdt>5)
    pdt = 5;
elseif (pdt<.1)
    pdt = .1;
end
set(handles.slider_pdt,'Value',pdt);
set(handles.edit_pdt,'String',pdt);

% Peak detection threshold
function edit_pdt_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Peak detection threshold
function slider_pdt_Callback(hObject, eventdata, handles)
pdt = get(handles.slider_pdt,'Value')
set(handles.edit_pdt,'String',num2str(pdt))
guidata(hObject,handles);

% Peak detection threshold
function slider_pdt_CreateFcn(hObject, eventdata, handles)
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end

% Spike removal
function checkbox_spikeremoval_Callback(hObject, eventdata, handles)
if get(hObject,'Value')
    set(handles.spikesens,'Visible','On')
    set(handles.senslabel,'Visible','On')
else
   set(handles.spikesens,'Visible','Off')
   set(handles.senslabel,'Visible','Off')
   
guidata(hObject,handles);
end

% Spike removal
function spikesens_Callback(hObject, eventdata, handles)
function spikesens_CreateFcn(hObject, eventdata, handles)
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end

% Polynomial order
function p_order_Callback(hObject, eventdata, handles)
function p_order_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Frame size
function f_size_Callback(hObject, eventdata, handles)
function f_size_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Post radius
function edit_postrad_Callback(hObject, eventdata, handles)
function edit_postrad_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Diastolic distance
function edit_dnum_Callback(hObject, eventdata, handles)
function edit_dnum_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Post deflection analysis
function checkbox_pda_Callback(hObject, eventdata, handles)

function edit_compfreq_Callback(hObject, eventdata, handles)
function edit_compfreq_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Autoscale
function checkbox_asy_Callback(hObject, eventdata, handles)

% Omit frequencies
function edit_omitfr_Callback(hObject, eventdata, handles)
function edit_omitfr_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Include frequencies
function edit_incl_fr_Callback(hObject, eventdata, handles)
function edit_incl_fr_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Show unfiltered data
function checkbox_plotraw_Callback(hObject, eventdata, handles)

% Save force trace figures
% --- Executes on button press in checkbox_saveftfig.
function checkbox_saveftfig_Callback(hObject, eventdata, handles)

% Save force time data
% --- Executes on button press in checkbox_saveftdata.
function checkbox_saveftdata_Callback(hObject, eventdata, handles)

% Start time
function edit_stime_Callback(hObject, eventdata, handles)
function edit_stime_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% End time
function edit_etime_Callback(hObject, eventdata, handles)
function edit_etime_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Include frequencies
function edit_inclfr_Callback(hObject, eventdata, handles)
function edit_inclfr_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit16_Callback(hObject, eventdata, handles)
function edit16_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function checkbox_asy_CreateFcn(hObject, eventdata, handles)
% hObject    handle to checkbox_asy (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called
