function [outputvars t2post_data ] = analyzePostDefl(post1def, post2def, time,thresh, p_order,w_size,tname)

post1def = -post1def;
post2def = post2def - min(post2def);
post1def = post1def - min(post1def);
post1deff = sgolayfilt(post1def,p_order,w_size);
post2deff = sgolayfilt(post2def,p_order,w_size);

assignin('base','p1',post1deff);
assignin('base','p2',post2deff);
assignin('base','time',time);

% get velocity information of posts
post1diff = diff(post1deff)./diff(time);
post2diff = diff(post2deff)./diff(time);
dtime = time(1:end-1);

% get coherence index from difference of two post velocity waveforms
postdiff = sum(abs((post1diff-post2diff).*dtime));
diffsum = sum(abs(post1diff).*dtime + abs(post2diff).*dtime);
coher_idx = 1 - postdiff/diffsum;

%thresholds for peak detection
stdd1 = thresh*std(post1deff);
stdd2 = thresh*std(post2deff);

% get peaks in each post
[x1 y1] = peakdet(post1deff, stdd1,time);
[x2 y2] = peakdet(post2deff, stdd2,time);

cxy = 0;
cyx = 0;

% get symmetrical and asymmetrical components Qt and qt

dlr = [];

for i = 2:length(x1)-1
    for k = 2:length(x2)-1
        
        dxy = x1(i,1)- x2(k,1);
        tauxy = min([x1(i,1)-x1(i-1,1),x1(i+1,1)-x1(i,1),x2(k,1)-x2(k-1,1),x2(k+1,1)-x2(k,1)])/2;
  
        if 0 <  dxy && dxy<tauxy
            jxy = 1;
            jyx = 0;
            dlr = [dlr;dxy];
        
        elseif dxy<0 && dxy>-tauxy
            jyx = 1;
            jxy = 0;
            dlr = [dlr;dxy];
                    
        elseif x1(i,1) == x2(k,1)
            jxy = .5;
            jyx = .5;
            dlr = [dlr;dxy];
             
        else
            jxy = 0;
            jyx = 0;
        end
        
        cxy = cxy + jxy;
        cyx = cyx + jyx;
    end
end

Qt = (cxy + cyx)/((length(x1)-2)*(length(x2)-2))^.5 ;
qt = (cyx - cxy)/((length(x1)-2)*(length(x2)-2))^.5 ;
lrt = mean(dlr);
LRMPD = mean(abs(dlr));
LRSPD = std(abs(dlr));

symm_idx = -lrt/LRMPD;

d1 = diff(x1(:,1));
d2 = diff(x2(:,1));

MNN_L = mean(d1);
MNN_R = mean(d2);
SDNN_L = std(d1);
SDNN_R = std(d2);

MSD_L = mean(abs(diff(d1)));
MSD_R = mean(abs(diff(d2)));
SDSD_L = std(abs(diff(d1)));
SDSD_R = std(abs(diff(d2)));

t2prev1 = d1(1:end-1);
t2post1 = d1(2:end);

t2prev2 = d2(1:end-1);
t2post2 = d2(2:end);

t2post_data = {[t2prev1 t2post1] [t2prev2 t2post2]};

figure
subplot(3,1,1)
hold on
plot(time,post1deff,'b')
plot(time,post2deff,'r')

plot(x1(:,1),x1(:,2),'g*')
plot(x2(:,1),x2(:,2),'go')

title(tname)

xlabel('time (s)')
ylabel('deflection (mm)')

m = 1.05*max([t2prev1;t2post1;t2prev2;t2post2]);

hold off
subplot(3,1,2)
plot(t2prev1,t2post1,'*');
hold on
plot(t2prev2,t2post2,'ro');
xlim([0 m])
ylim([0 m])

plot(0:.1:m,0:.1:m,'k')
xlabel('Time to Previous Peak (s)')
ylabel('Time to Next Peak (s)')

hold off

subplot(3,1,3)
plot(dtime,post1diff);
hold on

plot(dtime,post2diff,'r');
ylabel('Velocity (mm/s)')
xlabel ('time (s)')

hold off
[Qt, coher_idx, qt, symm_idx, MSD_L, MSD_R, SDSD_L, SDSD_R, LRMPD,LRSPD, MNN_L, MNN_R, SDNN_L, SDNN_R]
outputvars = [Qt, coher_idx, qt, symm_idx, MSD_L, MSD_R, SDSD_L, SDSD_R, LRMPD,LRSPD, MNN_L, MNN_R, SDNN_L, SDNN_R];

end


