function [slopes] = analyze_slopes(time, force, contractpoints, relaxpoints, plotslopes,hands )
% analyze_slopes.m find slopes of contractions and relaxations using points
% given indices in contract_points and relax_points and update intervention data

%   Looks at post_deflection data at the given indices and does a linear
%   fit to find average contraction and relaxation velocity between 10 and
%   90 percent developed force (or whatever indices are specified to).
%   Updates the intervention table data with mean and standard deviation of
%   velocity

if plotslopes == 1 % if a plot is wanted
    
    w = waitbar(0, 'Fitting slope points');
    for j = 1:length(contractpoints)
        h = hands(j);
        
        for i = 1:size(contractpoints{j},1)
            
            slopes{j}(i,1:2) = slopeFit(time{j}(contractpoints{j}{i,2}), force{j}(contractpoints{j}{i,2}),h);
            slopes{j}(i,3:4) = slopeFit(time{j}(relaxpoints{j}{i,2}), force{j}(relaxpoints{j}{i,2}),h);
            waitbar(i/length(contractpoints));
            
        end
    end
    
    % don't plot anything
else
    w = waitbar(0, 'Fitting slope points');
    
    for j = 1:length(contractpoints)
        for i = 1:length(contractpoints{j})
            
            slopes{j}(i,1:2) = slopeFit(time{j}(contractpoints{j}{i,2}), force{j}(contractpoints{j}{i,2}),0);
            slopes{j}(i,3:4) = slopeFit(time{j}(relaxpoints{j}{i,2}), force{j}(relaxpoints{j}{i,2}),0);
            waitbar(i/length(contractpoints));
            
        end
    end
    
end

close(w)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Function for linear fitting using curve fitting toolbox function
%    slopeF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [cfs] =  slopeFit(x,y,phandle)
%CREATEFIT Create plot of data sets and fits
%   CREATEFIT(X,Y)
%   Creates a plot, similar to the plot in the main Curve Fitting Tool,
%   using the data that you provide as input.  You can
%   use this function with the same data you used with CFTOOL
%   or with different data.  You may want to edit the function to
%   customize the code and this help message.
%
%   Number of data sets:  1
%   Number of fits:  1

% Data from data set "y vs. x":
%     X = x:
%     Y = y:
%     Unweighted

% Set up figure to receive data sets and fits

% --- Create fit "fit 1"

if length(x) >2
    ok_ = isfinite(x) & isfinite(y);
    if ~all( ok_ )
        warning( 'GenerateMFile:IgnoringNansAndInfs',...
            'Ignoring NaNs and Infs in data.' );
    end
    ft_ = fittype('poly1');
    
    % Fit this model using new data
    cf_ = fit(x(ok_),y(ok_),ft_);
    
    % Plot this fit
    cfs = coeffvalues(cf_);
    
    if phandle~=0
        axes(phandle)
        hold on;
        line(x,cfs(1)*x+cfs(2),'Color','r');
        legend off;
    end
    % Turn off legend created by plot method.
    
    % Store line handle and fit name for legend.
    
else
    cfs = NaN;
end

hold off;

end

